#ifndef __fileselect_h_

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include "gtweakui.h"

void fileselect(GtkWidget *w, gpointer d);
void dirselect(GtkWidget *w, gpointer d);

#define __fileselect_h_
#endif
